@@ -4,9 +4,7 @@ use memchr;
 
 use error::*;
 
-pub(crate) fn preamble_skipcount<R: Read>(reader: &mut R, n_preamble_rows: usize)
-    -> Result<usize>
-{
+pub(crate) fn preamble_skipcount<R: Read>(reader: &mut R, n_preamble_rows: usize) -> Result<usize> {
     if n_preamble_rows == 0 {
         return Ok(0);
     }
@@ -14,7 +12,9 @@ pub(crate) fn preamble_skipcount<R: Read>(reader: &mut R, n_preamble_rows: usize
     loop {
         let cap = 1 << 12;
         let mut buffer = vec![0; cap];
-        unsafe { buffer.set_len(cap); }
+        unsafe {
+            buffer.set_len(cap);
+        }
         let n_read = reader.read(&mut buffer)?;
         let mut crlf_pos = 0;
         let mut found = true;
@@ -22,7 +22,7 @@ pub(crate) fn preamble_skipcount<R: Read>(reader: &mut R, n_preamble_rows: usize
             match memchr::memchr(b'\n', &buffer[crlf_pos..]) {
                 Some(pos) => {
                     crlf_pos += pos + 1;
-                },
+                }
                 None => {
                     found = false;
                     break;
